<?php
/*
	Pull down NextBus data - Predictions
	Must be called AFTER getRouteConfig!
	Gets predictions, and messages.
	API Doc: http://www.nextbus.com/xmlFeedDocs/NextBusXMLFeed.pdf
*/

$agency = "rutgers";

// Required includes
require_once('../config.php');
require_once('../functions.php');

// Open SQL connection
$db = dbc();

// only need to update once a day. unless set otherwise in daemon.
if (!isset($getPredictions_waitSeconds)) $getPredictions_waitSeconds = 0;

// get timestamp when this selector was last queried
$lastFetched = 0;
$result = dbq($db, "SELECT `time` FROM `lastChecked` WHERE `selector` = 'predictions' LIMIT 1");
if ($result->num_rows == 1) {
	$arr = $result->fetch_array();
	$lastFetched = $arr[0];
}

$routes = array();
$q = "SELECT `routes`.`tag` AS `route`, `route_stops`.`stop` AS `stop` FROM `routes` LEFT JOIN `route_stops` ON `route_stops`.`route`=`routes`.`tag` AND `route_stops`.`agency` = `routes`.`agency` WHERE `routes`.`agency` = '{$agency}'";
$result = dbq($db, $q);
while ($route = $result->fetch_array()) $routes[$route['route']][$route['stop']] = $route['stop'];

$stopsPerRequest = 100;
$i = 0;
$urls = array();
$routeStops = array();
$urlBase = "http://webservices.nextbus.com/service/publicXMLFeed?command=predictionsForMultiStops&a={$agency}";

foreach ($routes as $route => $stops) {
	foreach ($stops as $stop) {
	
		// URL
		//  "http://webservices.nextbus.com/service/publicXMLFeed?command=predictions&a={$agency}&r={$route}&s={$stop}";
		$urls[floor($i / $stopsPerRequest)][] = "&stops={$route}|{$stop}";
		$routeStops[floor($i / $stopsPerRequest)][] = array($route,$stop);
		$i++;
	}
}

dbq($db, "LOCK TABLES `predictions` WRITE, `messages` WRITE, `lastChecked` WRITE");
dbq($db, "DELETE FROM `messages` WHERE `agency` = '{$agency}'");
dbq($db, "DELETE FROM `predictions` WHERE `agency` = '{$agency}'");

foreach ($urls as $key => $urlParams) {
		$url = $urlBase . implode("",$urlParams);
		
		// ask for new data, if needed
		if ($lastFetched + $getPredictions_waitSeconds < time()) {
			if ($source = file_get_contents($url)) {
				if ($thisXML = simplexml_load_string($source)) {
					
					foreach ($thisXML->predictions as $predictions) {
						
						foreach ($predictions as $k => $v) $predictions[$k] = $db->real_escape_string($v);
						
						// messages
						foreach ($predictions->message as $message) {
							$text = $db->real_escape_string($message['text']);
							$q = "INSERT INTO `messages` ";
							$q .= "(`agency`,`routeTag`,`stopTag`,`message`) ";
							$q .= "VALUES ('{$agency}','{$predictions['routeTag']}','{$predictions['stopTag']}','{$text}')";
							dbq($db, $q);
						}
					
						// predictions
						foreach ($predictions->direction as $direction) {
							foreach ($direction->prediction as $p) {
								foreach ($p as $k => $v) $p[$k] = $db->real_escape_string($v);
								$q = "INSERT INTO `predictions` ";
								$q .= "(`agency`,`routeTag`,`stopTag`,`epochTime`,`seconds`,`minutes`,`isDeparture`,`affectedByLayover`,`dirTag`,`vehicle`,`block`) ";
								$q .= "VALUES ('{$agency}','{$predictions['routeTag']}','{$predictions['stopTag']}','{$p['epochTime']}','{$p['seconds']}','{$p['minutes']}'";
								$q .= ",'{$p['isDeparture']}','{$p['affectedByLayover']}','{$p['dirTag']}','{$p['vehicle']}','{$p['block']}')";
								dbq($db, $q);
							}
						}
					
					}
					

					// update lastChecked.
					$tSec = time();
					dbq($db, "INSERT INTO `lastChecked` (`selector`, `time`) VALUES ('predictions', '" . $tSec . "') ON DUPLICATE KEY UPDATE `time` = VALUES(`time`)");
					
				} else $warning = "Could not parse NextBus data as XML";
			} else $warning = "Could not fetch NextBus data";
		}

}

dbq($db, "UNLOCK TABLES;");
$db->close();

?>